import { IAspect, IConstruct } from "@aws-cdk/core";
export interface FSBPConfig {
    iam?: {
        fullAdmin?: boolean;
        wildcardServiceActions?: boolean;
    };
    lambda?: {
        supportedRuntimes?: boolean;
    };
    rds?: {
        publicAccess?: boolean;
        storageEncrypted?: boolean;
        multiAz?: boolean;
        enhancedMonitoring?: boolean;
        deletionProtection?: boolean;
        logExports?: boolean;
        iamAuth?: boolean;
        autoMinorVersionUpgrades?: boolean;
        copyTagsToSnapshot?: boolean;
    };
    dynamodb?: {
        autoScaling?: boolean;
        pointInTimeRecovery?: boolean;
    };
}
/**
 * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html
 */
export declare class AWSFoundationalSecurityBestPracticesChecker implements IAspect {
    private Config;
    /**
     * Initialise a new Checker. All values in the configuration default to true if not provided.
     * @param config
     */
    constructor(config?: FSBPConfig);
    visit(node: IConstruct): void;
    private checkRDSCompliance;
    /**
     * [RDS.1] RDS snapshots should be private
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-1
     */
    private checkSnapshotPublicAccess;
    /**
     * [RDS.2] RDS DB instances should prohibit public access, determined by the PubliclyAccessible configuration
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-2
     */
    private checkRDSPublicAccess;
    /**
     * [RDS.3] RDS DB instances should have encryption at rest enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-3
     */
    private checkRDSEncryption;
    /**
     * [RDS.5] RDS DB instances should be configured with multiple Availability Zones
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-5
     */
    private checkMultipleAZs;
    /**
     * [RDS.6] Enhanced monitoring should be configured for RDS DB instances and clusters
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-6
     */
    private checkEnhancedMonitoring;
    /**
     * [RDS.8] RDS DB instances should have deletion protection enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-8
     */
    private checkDeletionProtection;
    /**
     * [RDS.9] Database logging should be enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-9
     */
    private checkLogExports;
    /**
     * [RDS.10] IAM authentication should be configured for RDS instances & [RDS.12] IAM authentication should be configured for RDS clusters
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-10 & https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-12
     */
    private checkIAMAuthentication;
    /**
     * [RDS.13] RDS automatic minor version upgrades should be enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-13
     */
    private checkMinorVersionUpgrades;
    /**
     * [RDS.16] RDS DB clusters should be configured to copy tags to snapshots & [RDS.17] RDS DB instances should be configured to copy tags to snapshots
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-16 & https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-17
     */
    private checkCopyTagsToSnapshot;
    private checkLambdaCompliance;
    /**
     * [Lambda.1] Lambda function policies should prohibit public access
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-1
     */
    private checkLambdaPublicAccess;
    private supportedLambdaRuntimes;
    /**
     * [Lambda.2] Lambda functions should use supported runtimes
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-2
     */
    private checkLambdaSupportedRuntime;
    private checkIAMPolicyCompliance;
    /**
     * [IAM.1] IAM policies should not allow full "*" administrative privileges.
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-1
     */
    private checkIAMPolicyFullAdmin;
    /**
     * [IAM.21] IAM customer managed policies that you create should not allow wildcard actions for services
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-21
     */
    private checkIAMPolicyServiceWildcards;
    private checkDynamoDBCompliance;
    /**
     * [DynamoDB.1] DynamoDB tables should automatically scale capacity with demand
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-dynamodb-1
     */
    private checkAutoScaling;
    /**
     * [DynamoDB.2] DynamoDB tables should have point-in-time recovery enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-dynamodb-2
     */
    private checkPointInTimeRecovery;
}
