"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AWSFoundationalSecurityBestPracticesChecker = void 0;
const core_1 = require("@aws-cdk/core");
const aws_iam_1 = require("@aws-cdk/aws-iam");
const aws_lambda_1 = require("@aws-cdk/aws-lambda");
const aws_rds_1 = require("@aws-cdk/aws-rds");
const aws_dynamodb_1 = require("@aws-cdk/aws-dynamodb");
const getDescriptor = (node) => {
    return node instanceof aws_rds_1.CfnDBInstance ? "instances" : "cluster";
};
/**
 * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html
 */
class AWSFoundationalSecurityBestPracticesChecker {
    /**
     * Initialise a new Checker. All values in the configuration default to true if not provided.
     * @param config
     */
    constructor(config = {
        iam: { fullAdmin: true, wildcardServiceActions: true },
        lambda: { supportedRuntimes: true },
        rds: {
            publicAccess: true,
            storageEncrypted: true,
            multiAz: true,
            enhancedMonitoring: true,
            deletionProtection: true,
            logExports: true,
            iamAuth: true,
            autoMinorVersionUpgrades: true,
            copyTagsToSnapshot: true,
        },
        dynamodb: { autoScaling: true, pointInTimeRecovery: true },
    }) {
        this.supportedLambdaRuntimes = [
            aws_lambda_1.Runtime.NODEJS_14_X,
            aws_lambda_1.Runtime.NODEJS_12_X,
            aws_lambda_1.Runtime.NODEJS_10_X,
            aws_lambda_1.Runtime.PYTHON_3_8,
            aws_lambda_1.Runtime.PYTHON_3_7,
            aws_lambda_1.Runtime.PYTHON_3_6,
            aws_lambda_1.Runtime.RUBY_2_7,
            aws_lambda_1.Runtime.RUBY_2_5,
            aws_lambda_1.Runtime.JAVA_11,
            aws_lambda_1.Runtime.JAVA_8,
            aws_lambda_1.Runtime.JAVA_8_CORRETTO,
            aws_lambda_1.Runtime.GO_1_X,
            aws_lambda_1.Runtime.DOTNET_CORE_3_1,
            aws_lambda_1.Runtime.DOTNET_CORE_2_1,
        ];
        this.Config = config;
    }
    visit(node) {
        if (node instanceof aws_iam_1.CfnPolicy) {
            this.checkIAMPolicyCompliance(node);
        }
        else if (node instanceof aws_lambda_1.CfnFunction) {
            this.checkLambdaCompliance(node);
        }
        else if (node instanceof aws_rds_1.CfnDBInstance || node instanceof aws_rds_1.CfnDBCluster) {
            this.checkRDSCompliance(node);
        }
        else if (node instanceof aws_dynamodb_1.CfnTable) {
            this.checkDynamoDBCompliance(node);
        }
    }
    checkRDSCompliance(node) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t;
        this.checkSnapshotPublicAccess(node);
        if ((_b = (_a = this.Config.rds) === null || _a === void 0 ? void 0 : _a.publicAccess) !== null && _b !== void 0 ? _b : true) {
            this.checkRDSPublicAccess(node);
        }
        if ((_d = (_c = this.Config.rds) === null || _c === void 0 ? void 0 : _c.storageEncrypted) !== null && _d !== void 0 ? _d : true) {
            this.checkRDSEncryption(node);
        }
        if ((_f = (_e = this.Config.rds) === null || _e === void 0 ? void 0 : _e.multiAz) !== null && _f !== void 0 ? _f : true) {
            this.checkMultipleAZs(node);
        }
        if ((_h = (_g = this.Config.rds) === null || _g === void 0 ? void 0 : _g.enhancedMonitoring) !== null && _h !== void 0 ? _h : true) {
            this.checkEnhancedMonitoring(node);
        }
        if ((_k = (_j = this.Config.rds) === null || _j === void 0 ? void 0 : _j.deletionProtection) !== null && _k !== void 0 ? _k : true) {
            this.checkDeletionProtection(node);
        }
        if ((_m = (_l = this.Config.rds) === null || _l === void 0 ? void 0 : _l.logExports) !== null && _m !== void 0 ? _m : true) {
            this.checkLogExports(node);
        }
        if ((_p = (_o = this.Config.rds) === null || _o === void 0 ? void 0 : _o.iamAuth) !== null && _p !== void 0 ? _p : true) {
            this.checkIAMAuthentication(node);
        }
        if ((_r = (_q = this.Config.rds) === null || _q === void 0 ? void 0 : _q.autoMinorVersionUpgrades) !== null && _r !== void 0 ? _r : true) {
            this.checkMinorVersionUpgrades(node);
        }
        if ((_t = (_s = this.Config.rds) === null || _s === void 0 ? void 0 : _s.copyTagsToSnapshot) !== null && _t !== void 0 ? _t : true) {
            this.checkCopyTagsToSnapshot(node);
        }
    }
    /**
     * [RDS.1] RDS snapshots should be private
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-1
     */
    checkSnapshotPublicAccess(node) {
        // TODO: This is complicated...
    }
    /**
     * [RDS.2] RDS DB instances should prohibit public access, determined by the PubliclyAccessible configuration
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-2
     */
    checkRDSPublicAccess(node) {
        // This check is only applicable to DB Instances.
        if (node instanceof aws_rds_1.CfnDBInstance && node.publiclyAccessible) {
            // Undefined is the same as false in this context, so we don't need to check or raise an error.
            core_1.Annotations.of(node).addError("[RDS.2] RDS DB instances should prohibit public access, determined by the PubliclyAccessible configuration");
        }
    }
    /**
     * [RDS.3] RDS DB instances should have encryption at rest enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-3
     */
    checkRDSEncryption(node) {
        // This check is only applicable to instances, not clusters.
        if (node instanceof aws_rds_1.CfnDBInstance &&
            node.dbClusterIdentifier === undefined &&
            !node.storageEncrypted) {
            core_1.Annotations.of(node).addError(`[RDS.3] RDS DB ${getDescriptor(node)} should have encryption at rest enabled`);
        }
    }
    /**
     * [RDS.5] RDS DB instances should be configured with multiple Availability Zones
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-5
     */
    checkMultipleAZs(node) {
        // This check is only applicable to instances, not clusters.
        if (node instanceof aws_rds_1.CfnDBInstance &&
            node.dbClusterIdentifier === undefined &&
            !node.multiAz) {
            core_1.Annotations.of(node).addError(`[RDS.5] RDS DB ${getDescriptor(node)} should be configured with multiple Availability Zones`);
        }
    }
    /**
     * [RDS.6] Enhanced monitoring should be configured for RDS DB instances and clusters
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-6
     */
    checkEnhancedMonitoring(node) {
        if (node instanceof aws_rds_1.CfnDBInstance &&
            (!node.monitoringInterval || node.monitoringInterval < 1)) {
            // Each instance in a cluster requires enhanced monitoring.
            core_1.Annotations.of(node).addError("[RDS.6] Enhanced monitoring should be configured for RDS DB instances and clusters");
        }
    }
    /**
     * [RDS.8] RDS DB instances should have deletion protection enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-8
     */
    checkDeletionProtection(node) {
        if (node instanceof aws_rds_1.CfnDBCluster && !node.deletionProtection) {
            core_1.Annotations.of(node).addError(`[RDS.8] RDS DB ${getDescriptor(node)} should have deletion protection enabled`);
        }
        else if (node instanceof aws_rds_1.CfnDBInstance &&
            node.dbClusterIdentifier === undefined &&
            !node.deletionProtection) {
            core_1.Annotations.of(node).addError(`[RDS.8] RDS DB ${getDescriptor(node)} should have deletion protection enabled`);
        }
    }
    /**
     * [RDS.9] Database logging should be enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-9
     */
    checkLogExports(node) {
        if (node instanceof aws_rds_1.CfnDBCluster ||
            (node instanceof aws_rds_1.CfnDBInstance && node.dbClusterIdentifier === undefined)) {
            if (node.enableCloudwatchLogsExports === undefined ||
                node.enableCloudwatchLogsExports.length === 0) {
                core_1.Annotations.of(node).addError("[RDS.9] Database logging should be enabled");
            }
            else {
                // Check that the log types are valid.
                let expectedLogTypes = [];
                switch (node.engine) {
                    case "mysql":
                    case "aurora":
                    case "aurora-mysql":
                    case "mariadb":
                        expectedLogTypes = ["audit", "error", "general", "slowquery"];
                        break;
                    case "oracle-ee":
                    case "oracle-se2":
                        expectedLogTypes = ["alert", "audit", "trace", "listener"];
                        break;
                    case "postgres":
                    case "aurora-postgres":
                        expectedLogTypes = ["postgresql", "upgrade"];
                        break;
                    case "sqlserver-ee":
                    case "sqlserver-ex":
                    case "sqlserver-se":
                    case "sqlserver-web":
                        expectedLogTypes = ["error", "agent"];
                        break;
                    default:
                        expectedLogTypes = ["invalid"];
                        break;
                }
                if (!node.enableCloudwatchLogsExports.every((log) => expectedLogTypes.includes(log))) {
                    core_1.Annotations.of(node).addError("[RDS.9] Database logging should be enabled");
                }
            }
        }
    }
    /**
     * [RDS.10] IAM authentication should be configured for RDS instances & [RDS.12] IAM authentication should be configured for RDS clusters
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-10 & https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-12
     */
    checkIAMAuthentication(node) {
        if (node instanceof aws_rds_1.CfnDBInstance &&
            node.dbClusterIdentifier === undefined &&
            !node.enableIamDatabaseAuthentication) {
            core_1.Annotations.of(node).addError("[RDS.10] IAM authentication should be configured for RDS instances");
        }
        else if (node instanceof aws_rds_1.CfnDBCluster &&
            !node.enableIamDatabaseAuthentication) {
            core_1.Annotations.of(node).addError("[RDS.12] IAM authentication should be configured for RDS clusters");
        }
    }
    /**
     * [RDS.13] RDS automatic minor version upgrades should be enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-13
     */
    checkMinorVersionUpgrades(node) {
        if (node instanceof aws_rds_1.CfnDBInstance && !node.autoMinorVersionUpgrade) {
            core_1.Annotations.of(node).addError("[RDS.13] RDS automatic minor version upgrades should be enabled");
        }
    }
    /**
     * [RDS.16] RDS DB clusters should be configured to copy tags to snapshots & [RDS.17] RDS DB instances should be configured to copy tags to snapshots
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-16 & https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-rds-17
     */
    checkCopyTagsToSnapshot(node) {
        if (!node.copyTagsToSnapshot) {
            if (node instanceof aws_rds_1.CfnDBCluster) {
                core_1.Annotations.of(node).addWarning("[RDS.16] RDS DB clusters should be configured to copy tags to snapshots");
            }
            else if (node instanceof aws_rds_1.CfnDBInstance &&
                node.dbClusterIdentifier === undefined) {
                core_1.Annotations.of(node).addWarning("[RDS.17] RDS DB instances should be configured to copy tags to snapshots");
            }
        }
    }
    checkLambdaCompliance(node) {
        var _a, _b;
        this.checkLambdaPublicAccess(node);
        if ((_b = (_a = this.Config.lambda) === null || _a === void 0 ? void 0 : _a.supportedRuntimes) !== null && _b !== void 0 ? _b : true) {
            this.checkLambdaSupportedRuntime(node);
        }
    }
    /**
     * [Lambda.1] Lambda function policies should prohibit public access
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-1
     */
    checkLambdaPublicAccess(node) {
        // TODO: This is complicated...
    }
    /**
     * [Lambda.2] Lambda functions should use supported runtimes
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-lambda-2
     */
    checkLambdaSupportedRuntime(node) {
        if (!this.supportedLambdaRuntimes.some((r) => r.name === node.runtime)) {
            core_1.Annotations.of(node).addError("[Lambda.2] Lambda functions should use supported runtimes");
        }
    }
    checkIAMPolicyCompliance(node) {
        var _a, _b, _c, _d;
        if ((_b = (_a = this.Config.iam) === null || _a === void 0 ? void 0 : _a.fullAdmin) !== null && _b !== void 0 ? _b : true) {
            this.checkIAMPolicyFullAdmin(node);
        }
        if ((_d = (_c = this.Config.iam) === null || _c === void 0 ? void 0 : _c.wildcardServiceActions) !== null && _d !== void 0 ? _d : true) {
            this.checkIAMPolicyServiceWildcards(node);
        }
    }
    /**
     * [IAM.1] IAM policies should not allow full "*" administrative privileges.
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-1
     */
    checkIAMPolicyFullAdmin(node) {
        if (node.policyDocument.hasOwnProperty("statements")) {
            const statements = node.policyDocument.statements;
            if (statements.some((s) => s.effect === aws_iam_1.Effect.ALLOW &&
                s.action.some((a) => a === "*") &&
                s.resource.some((r) => r === "*"))) {
                core_1.Annotations.of(node).addError('[IAM.1] IAM policies should not allow full "*" administrative privileges.');
            }
        }
    }
    /**
     * [IAM.21] IAM customer managed policies that you create should not allow wildcard actions for services
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-iam-21
     */
    checkIAMPolicyServiceWildcards(node) {
        if (node.policyDocument.hasOwnProperty("statements")) {
            const statements = node.policyDocument.statements;
            if (statements.some((s) => s.effect === aws_iam_1.Effect.ALLOW &&
                s.action.some((a) => a.endsWith(":*")))) {
                core_1.Annotations.of(node).addError("[IAM.21] IAM customer managed policies that you create should not allow wildcard actions for services.");
            }
        }
    }
    checkDynamoDBCompliance(node) {
        var _a, _b, _c, _d;
        if ((_b = (_a = this.Config.dynamodb) === null || _a === void 0 ? void 0 : _a.autoScaling) !== null && _b !== void 0 ? _b : true) {
            this.checkAutoScaling(node);
        }
        if ((_d = (_c = this.Config.dynamodb) === null || _c === void 0 ? void 0 : _c.pointInTimeRecovery) !== null && _d !== void 0 ? _d : true) {
            this.checkPointInTimeRecovery(node);
        }
    }
    /**
     * [DynamoDB.1] DynamoDB tables should automatically scale capacity with demand
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-dynamodb-1
     */
    checkAutoScaling(node) {
        // Look for ScalableTableAttribute or ScalableTarget
        if (node.node.scope instanceof aws_dynamodb_1.Table) {
            const table = node.node.scope;
            if (table.hasOwnProperty("tableScaling")) {
                const tableScaling = table["tableScaling"];
                if (!tableScaling.hasOwnProperty("scalableReadAttribute") ||
                    !tableScaling.hasOwnProperty("scalableWriteAttribute")) {
                    // If it's missing a scalable attribute, it's not a scalable table.
                    core_1.Annotations.of(node).addError("[DynamoDB.1] DynamoDB tables should automatically scale capacity with demand");
                }
            }
            else {
                // If it's missing tableScaling, it's not a scalable table.
                core_1.Annotations.of(node).addError("[DynamoDB.1] DynamoDB tables should automatically scale capacity with demand");
            }
        }
    }
    /**
     * [DynamoDB.2] DynamoDB tables should have point-in-time recovery enabled
     * Ref: https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-standards-fsbp-controls.html#fsbp-dynamodb-2
     */
    checkPointInTimeRecovery(node) {
        if (!node.pointInTimeRecoverySpecification) {
            // Nothing explicit about PITR being on or off, raise an error.
            core_1.Annotations.of(node).addError("[DynamoDB.2] DynamoDB tables should have point-in-time recovery enabled");
        }
        else {
            if (!(0, core_1.isResolvableObject)(node.pointInTimeRecoverySpecification) &&
                !node.pointInTimeRecoverySpecification.pointInTimeRecoveryEnabled) {
                // PITR is explicitly off, raise an error.
                core_1.Annotations.of(node).addError("[DynamoDB.2] DynamoDB tables should have point-in-time recovery enabled");
            }
        }
    }
}
exports.AWSFoundationalSecurityBestPracticesChecker = AWSFoundationalSecurityBestPracticesChecker;
